% Create a field from quantities at integration points.
%
% function fld = field_from_integration_points(self, ...
%         geom, T, output, component)
%
% Input arguments
%     geom     - reference geometry field
%     T        - temperature field
%     output   - this is what you would assign to the 'output' attribute of
%                the context argument of the material update() method.
%     component- component of the 'output' array: see the material update()
%                method.
% Output argument
%     fld - the new field that can be used to map values the colors
%
function fld = field_from_integration_points(self, geom, T, output, component)
    gcells = get(self.feblock,'gcells');
    % Make the inverse map from finite element nodes to gcells
    f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
    gmap=get (f2gmap,'gcell_map');
    % Container of intermediate results
    sum_inv_dist =0*(1:length(gmap))';
    sum_quant_inv_dist =zeros(length(gmap),length(component));
    fld =field(struct ('name',['fld'], 'dim', length(component), 'data',zeros(get(T,'nfens'),length(component))));
    nvals = gather (fld, (1: get(fld,'nfens')),'values','noreshape');
    % This is an inverse-distance interpolation inspector.
    x= []; conn = [];
    function idat=idi(idat, out, xyz, pc)
        d=x-ones(length(conn),1)*xyz;
        ld=reshape(d',prod(size(d)),1);
        d =sum(reshape(ld.*ld,size(d,2), size(d,1)));
        zi = find(d==0);
        if (~isempty(zi))
            nzi = find(d>0);
            d(zi) =min(d(nzi))/1e9;
        end
        invd =1./d;
        quant=reshape(out(component),1,length(component));
        sum_quant_inv_dist(conn,:)=sum_quant_inv_dist(conn,:) + invd'*quant;
        sum_inv_dist(conn)=sum_inv_dist(conn)+invd';
    end
      conns = get(gcells, 'conn'); % connectivity
  % Loop over cells to interpolate to nodes
    idat=0;
    for i=1:count(gcells)
        conn = conns(i,:);
        x=gather(geom,conn,'values','noreshape');
        idat = inspect_integration_points(self, geom, T, ...
            i, struct ('output',output), @idi, idat);
    end
    % compute the data array
    nzi = find (sum_inv_dist);
    for j=1:length(component)
        nvals(nzi,j)=sum_quant_inv_dist(nzi,j)./sum_inv_dist(nzi);
    end
    % Make the field
    fld = scatter(fld, (1: get(fld,'nfens')),nvals);
    return;
end
