% Transformation from local Cartesian displacements to natural deformations 
% of a beam element. 
%
% function aN = local_cartesian_to_natural (self, L)
%
% Matrix defined in Eq (4.8) of COMPUTER  METHODS  IN APPLIED  MECHANICS  AND  ENGINEERING   14  (1978)  401-451
% ON LARGE  DISPLACEMENT-SMALL   STRAIN  ANALYSIS   OF  STRUCTURES
% WITH  ROTATIONAL   DEGREES  OF  FREEDOM.
% J.H.  ARGYRIS,   P.C.  DUNNE  and  D.W. SCHARPF
% 
% Inputs:
% L= current length of the element
% 
% Outputs:
% aN= transformation matrix to take Cartesian (local) displacement increments in the 
%      element frame and to produce increments of natural deformations; 
%      see local_frames() for the definition of the natural deformations 
function aN = local_cartesian_to_natural (self, L)
    aN=[[ -1,   0,   0,  0,  0, 0, 1,    0,    0, 0,  0,  0]
        [  0,   0,   0,  0,  0, 1, 0,    0,    0, 0,  0, -1]
        [  0, 2/L,   0,  0,  0, 1, 0, -2/L,    0, 0,  0,  1]
        [  0,   0,   0,  0, -1, 0, 0,    0,    0, 0,  1,  0]
        [  0,   0, 2/L,  0, -1, 0, 0,    0, -2/L, 0, -1,  0]
        [  0,   0,   0, -1,  0, 0, 0,    0,    0, 1,  0,  0]];
end
% 
% syms L real
% [1,0,0,0,0,0;
%         0,0,0,0,1,-1;
%         0,0,0,0,1,1;
%         0,0,-1,1,0,0;
%         0,0,-1,-1,0,0;
%         0,1,0,0,0,0;
%         ]* [-1,0,0,0,0,0,1,0,0,0,0,0;
%         0,0,0,-1,0,0,0,0,0,1,0,0;
%         0,0,-1/L,0,1,0,0,0,1/L,0,0,0;
%         0,0,-1/L,0,0,0,0,0,1/L,0,1,0;
%         0,1/L,0,0,0,1,0,-1/L,0,0,0,0;
%         0,1/L,0,0,0,0,0,-1/L,0,0,0,1;
%         ]