% Compute the natural stiffness matrix.
%
% function DN= natural_stiffness(self, E, G, A, I2, I3, J, L) 
%
% Inputs:
% E, G= Young's and shear modulus, 
% A= cross-sectional area, 
% I2, I3=central moment of inertia of the cross-section about the x2 and x3 
% coordinate axis, 
% J=St Venant torsion constant, 
% L= current length of the element, 
% 
% Outputs:
% DN = 6 x 6 natural stiffness matrix
% 
function DN= natural_stiffness(self, E, G, A, I2, I3, J, L)
    DN= diag([E*A,
        E*I3,
        3*E*I3,
        E*I2,
        3*E*I2,
        G*J ]/L);
end