% Compute the stabilization stiffness matrices of the individual entities in the block
% (in this case nodes).
% 
% 
% 
% Return  element matrix set so they may be assembled.
%    Call as
% function ems = stiffness (self, geom, u)
%     geom=geometry field
%     u=displacement field
%
function ems = stab_stiffness (self, geom, u)
    gcells = get(self.feblock,'gcells');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts=get(integration_rule, 'npts');
    for j=1:size(pc,1)
        N{j} = bfun(gcells, pc(j,:));
        Nder{j} = bfundpar(gcells, pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    D = tangent_moduli(mat,struct([]));
    [Princd,W]=eig(D);
    [ignore,ix]=sort(diag(W));
    Wmean=(W(ix(1),ix(1))+W(ix(2),ix(2)))/2;
    if Wmean<=0
        Wmean=max(diag(W));
    end
    % Make the inverse map from finite element nodes to geometric cells
    f2gmap=fenode_to_gcell_map (struct ('gcells',gcells));
    gmap=get (f2gmap,'gcell_map');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Kes= cell(length(gmap),1);
    eqnums = cell(length(gmap),1);
    for i=1:length(gmap)
        eqnums{i} =[];
        Kes{i} = [];
    end
    xs =gather(geom, 1:length(gmap), 'values', 'noreshape'); % coordinates of all nodes
    eqnumss =gather(u, 1:length(gmap), 'eqnums','noreshape');% equation numbers for all nodes
    % Now loop over all finite element nodes
    lnmap =0*(1:length(gmap));
    patchconn = [];
    iem =1;
    for nix=1:length(gmap)
        gl=gmap{nix};
        thisnn=nix;
        if ~isempty(gl) % This node has a element patch in this block
            % establish local numbering of all nodes of the patch @node nix
            kconns = conns(gl,:); % connectivity
            patchconn = patch_conn (self,kconns,thisnn);
            np =length(patchconn);
            lnmap(patchconn) =1:np;% now store the local numbers
            c = xs(thisnn, :); % coordinates of central node

            lx = xs(patchconn,:); % coordinates of nodes
            Phi =self.hPhi(self,dim,np,lx,c);
            lastwarn('','') 
            A1 =Phi* ((Phi'*Phi)\Phi');
            [msgstr, msgid] = lastwarn;
            if (isempty(msgid))
                Kstab=self.stabfact*Wmean*(eye(size(A1))-A1);
                Kes{iem} =  Kstab;
                eqnums{iem} =  reshape(eqnumss(patchconn,:)',(np)*size(eqnumss,2),1);
                iem=iem+1;
            else
                disp(msgid)
            end
        end
        lnmap(patchconn) = 0;
    end
    %     If there are any interior integration points, execute the loop over
    %     all the cells and perform the integrations
    if (npts > size(conns,1))
        error(' Not implemented yet')
%         ems(length(gmap)+1:length(gmap)+length(gcells)) = deal(elemat);
%         Ke = zeros(get(x,'dim')*nfens); 
%         for i=1:length(gcells)
%             conn = get(gcells(i), 'conn'); % connectivity
%             lx = gather(x, conn, 'values', 'noreshape'); % coord
%             Ke = 0*Ke; % zero out element stiffness
%             % Loop over all integration points
%             for j=nconn+1:npts
%                 N = bfun(gcells(i),pc(j,:));

%                 detJ = Jacobian_volume(gcells(i),pc(j,:),lx);
%                 B = Blmat(gcells(i), pc(j,:), lx, Rm);
%                 D = tangent_moduli(mat,struct('xyz',N'*lx));
%                 Ke = Ke + B'*D*B * detJ * w(j);
%             end
%             Ke = set(ems(iem), 'mat', Ke);
%             eqnums{iem} = gather(u,conn,'eqnums');
%             iem=iem+1;
%         end
    end
    ems  = elematset(struct('mat',{Kes(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
end


