% Constructor of small-strain nonlinear Finite element block.
%    This class implements small displacement, small strain mechanics.
%    See the feblock constructor.
%
function retobj = feblock_defor_ss_nonlinear (varargin)
    class_name='feblock_defor_ss_nonlinear';
    if (nargin == 0)
        parent = feblock_defor_ss;
        self.matstates = [];
        self.Rm= [];
        self.N = {};
        self.Nder = {};
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            parent = feblock_defor_ss(varargin{:});
            self.matstates = [];
            gcells = get(parent,'gcells');
            if (count(gcells) < 1)
                error('Need at least one gcell!');
            end
            integration_rule = get(parent,'integration_rule');
            if ~isempty(integration_rule)
                npts_per_gcell = get(integration_rule,'npts_per_gcell');
                self.matstates = cell(count(gcells),npts_per_gcell);
                for i=1:count(gcells)
                    for j=1:npts_per_gcell;
                        self.matstates{i,j} = newmatstate(get(parent,'mater'));
                    end
                end
            else
                self.matstates = [];
            end
            integration_rule = get(parent, 'integration_rule');
            pc = get(integration_rule, 'param_coords');
            w  = get(integration_rule, 'weights');
            npts_per_gcell = get(integration_rule, 'npts');
            for j=1: npts_per_gcell
                self.N{j} = bfun (gcells, pc(j,:));
                self.Nder{j} = bfundpar (gcells, pc(j,:));
            end
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
