% Compute the element load vectors corresponding to nonzero
%
% function evs = nz_ebc_loads_surface_transfer(self, geom, temp)
%
% essential boundary
% conditions (prescribed temperature).
% Return an array of them so they may be assembled.
%    Call as
% evs = nz_ebc_loads_surface_transfer(feb, geom, temp)
%     geom=geometry field
%     temp=temperature field
%
function evs = nz_ebc_loads_surface_transfer(self, geom, temp)
    gcells = get(self.feblock,'gcells');
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    % Now loop over all gcells in the block
    j=1;
    for i=1:count(gcells)
        conn = conns(i,:); % connectivity
        pT = gather(temp, conn, 'prescribed_values');
        if norm (pT) ~= 0
            self=set(self, 'gcells',subset(gcells,i));
            ems = surface_transfer(self, geom, temp);
            mat=get(ems,'mat');
            Ke = mat{1}; % element matrix
            Fe{j} =  -Ke*pT;
            eqnums{j} = gather(temp, conn, 'eqnums');
            j=j+1;
        end
    end
    Fe=Fe(1:j-1);
    eqnums=eqnums(1:j-1);
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
 % The old way; About an order of magnitude slower than the above.
 %  ems = surface_transfer(self, geom, temp);
 %     gcells = get(self,'gcells');
 %     conns = get(gcells, 'conn'); % connectivity
 %     Fe = cell(size(conns,1),1);
 %     eqnums = cell(size(conns,1),1);
 %     for i=1:size(conns,1)
 %         eqnums{i} =gather(temp,conns(i,:),'eqnums');
 %         Fe{i} =zeros(length(eqnums{i}),1);
 %     end
 %     mat=get(ems,'mat');
 %     for i=1:size(conns,1)
 %         conn = conns(i,:); % connectivity
 %         pT = gather(temp, conn, 'prescribed_values');
 %         if norm (pT) ~= 0
 %             Fe{i} =-mat{i}*pT;
 %         end
 %     end
 %     evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
 % end 
