% Compute the surface heat transfer matrices of the individual gcells.
%
% function ems = surface_transfer(self, geom, temp)
%
% Return an array of the element matrices so they may be assembled.
%    Call as     ems = surface_transfer(feb, geom, temp)
%    where    geom=geometry field
%             temp=temperature field
function ems = surface_transfer(self, geom, temp)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % surface transfer coefficient
    h = self.surface_transfer;
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Ke{i} =zeros(nfens);
    end
    xs =get(geom,'values');
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_surface(gcells,conn, Ns{j}, J, x);
            Ke{i} = Ke{i} + ((h*Jac*w(j))*Ns{j})*Ns{j}';
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
