% Evaluate the surface Jacobian.
%
% function Jac = Jacobian_surface(self, conn, N, J, x)
%
%
%   For the zero-dimensional cell, the surface Jacobian is
%       (i) the product of the 0-D Jacobian and the other dimension (Units
%       of length squared);
%       or, when used as axially symmetric
%       (ii) the product of the 0-D Jacobian and the circumference of
%       the circle through the point pc and the other dimension (units of
%       length)
%   
%   where
%      self=gcellset,
%      conn=connectivity of a single cell in which the Jacobian is to be evaluated
%      N=values of the basis functions, size(N) = [nbfuns,1]
%      J = Jacobian matrix
%      x=array of spatial coordinates of the nodes, size(x) = [nbfuns,dim]
%
function Jac = Jacobian_surface(self, conn, N, J, x)
    if self.axisymm
        xyz =N'*x;
        Jac = 1*2*pi*xyz(1)*other_dimension(self, conn, N, x);
    else
        Jac = 1*other_dimension(self, conn, N, x);
    end
end
