% Evaluate the derivatives of the basis function matrix.
%
% function val = bfundpar (self, param_coords)
%
% Returns an array of NFENS rows, and DIM columns, where
%    NFENS=number of nodes, and
%    DIM=number of spatial dimensions.
% Call as:
%    Nder = bfundpar(g, pc)
% where g=gcell
%       pc=parametric coordinates, -1 < pc(j) < 1, length(pc)=3.
% Column j holds derivatives of the basis functions with respect
% to the parametric coordinate j. Row i holds derivatives of the basis
% function i with respect to all the parametric coordinates in turn.
%
function val = bfundpar (self, param_coords)
    xi=param_coords(1);
    xis = [(xi-1)*xi/2;  (xi+1)*xi/2;  -(xi+1)*(xi-1)];
    dxis = [(xi-1/2); (xi+1/2); -2*xi];
    eta=param_coords(2);
    etas = [(eta-1)*eta/2;  (eta+1)*eta/2;  -(eta+1)*(eta-1)];
    detas=[(eta-1/2); (eta+1/2); -2*eta];
    dxisetas=(dxis*etas');
    xisdetas=(xis*detas');
    val =  [dxisetas([1     2     5     4     3     8     6     7     9])',...
            xisdetas([1     2     5     4     3     8     6     7     9])'];
    return;
end
