% Deformable nonlinear hyperelastic isotropic
% materials of the neohookean type.
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options:
% required fields
%     E=Young's modulus
%     nu=Poisson ratio
%
function retobj = mater_defor_neohook (varargin)
    class_name='mater_defor_neohook';
    if (nargin == 0)
        parent=mater_defor;
        self.property = [];
        self.E = 1.0;
        self.nu = 0.0;
        retobj = class(self,class_name, parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            parent=mater_defor(options);
            self.property = options.property;
            self.E = get(self.property,'E');
            self.nu = get(self.property,'nu');
            retobj = class(self,class_name, parent);
            if (self.nu < 0)
                error('Negative Poisson ratio!');
            elseif (self.nu >= 0.5)
                error('Incompressible material!');
            end
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
