% Calculate the material stiffness matrix.
%   Call as:
%     D = tangent_moduli(m, context)
%   where
%     m=material
%    context=struct
% with fields
%  - mandatory
%  - optional
%     stiff_type=type of the stiffness,
%        either 'Lagrangean'
%        or 'Eulerian'
%              requires field 'F'= current deformation gradient
% the output arguments are
%     D=matrix 6x6
%
function D = tangent_moduli(self, context)
    stiff_type='Eulerian';
    if isfield(context,'stiff_type')
        stiff_type= context.stiff_type;
    end
    switch (stiff_type)
        case 'Eulerian'
            E = self.E;
            nu = self.nu;
            J =det(context.F);
            lambda = E * nu / (1 + nu) / (1 - 2*(nu));          % Lame constant #1
            mu     = E / (2 * (1 + nu));                        % shear modulus
            kappa  = E / 3 / (1 - 2*nu);                        % bulk modulus
            mI = diag([1 1 1 0.5 0.5 0.5]);
            m1     = [1 1 1 0 0 0]';
            D = (lambda / J) * m1 * m1' + 2 * (mu - lambda*log(J))/J * mI;
            return;
        otherwise
            error('Cannot handle');
            D=[]; % return non-usable value
            return;
    end
end
