% make_graphics_rep: Make graphics representation of data at the integration
%                    point.
%
%  Call as:
%    pp = make_graphics_rep(self, x, ms, F1, F, what_to_show), or
%    pp = make_graphics_rep(self, x, ms, F1, F, what_to_show, data)
%  where
%    x             - center
%    ms            - material state
%    F1            - deformation gradient at time t_n+1
%    F             - deformation gradient at time t_n
%    what_to_show  - string: name of the quantity to display
%                 if what_to_show=='stress'
%                    the graphics representation is an ellipsoid of radii corresponding
%                    to the principal values of stress; 
%                 otherwise
%                    the graphics representation is a sphere of radius corresponding
%                    to the value 
%    varargin=if supplied, data array, size(varargin)=[1,anyn] 
%                 if what_to_show=='stress'
%                    the data is expanded by six columns corresponding to 
%                    to the values of stress components (stress tensor is assumed to be symmetric!) 
%                 otherwise
%                    the data is expanded by one column corresponding to 
%                    to the quantity to plot
%   
% This function is being used by graphics_model.
%
function pp = make_graphics_rep (self, x, ms, F1, F, what_to_show, varargin)
  if (nargin < 7)
    data = [];
  else
    data = varargin{1};
  end
  [sigma,ignoreme] = update(self, ms, F1, F, 'Cauchy');
  switch (what_to_show)
    case 'stress'
      [V,D]=eig(stress_6_vector_to_tensor(self,sigma));
      data = [ data sigma' ];
      pp = graphics_rep_ellipsoid (x, V, diag(D), data);
    case 'equiv_pl_def'
      v = matstate_get(self, ms, 'equiv_pl_def');
      data = [ data v ];
      pp = graphics_rep_ellipsoid (x, eye(3,3), v * ones(3,1), data);
    otherwise
      data = data;
  end
  return;

