% Calculate the material stiffness matrix.
%   Call as:
%     D = tangent_moduli(m, context)
%   where
%     m=material
%    context=struct
% with fields
%  - mandatory
%  - optional
%     stiff_type=type of the stiffness,
%        either 'Lagrangean'
%        or 'Eulerian'
%              requires field 'F'= current deformation gradient
% the output arguments are
%     D=matrix 6x6
%
%   where
%     D=matrix 6x6
%     m=material
%     ms=material state
%     F1=deformation gradient at time t+dt
%     F=deformation gradient at time t
%     stiff_type=stiffness type ('Lagrangean', 'Eulerian')
%
function D = tangent_moduli(self, context)
    E = get(self.property,'E', context);;
    nu = get(self.property,'nu', context);;
    sigma_y = get(self.property,'sigma_y', context);;
    lambda = E * nu / (1 + nu) / (1 - 2*(nu));          % Lame constant #1
    mu     = E / (2 * (1 + nu));                        % shear modulus
    kappa  = E / 3 / (1 - 2*nu);                        % bulk modulus
    mI = diag([1 1 1 0.5 0.5 0.5]);
    m1     = [1 1 1 0 0 0]';
    eps1 = context.strain;                        % total strain
    e1   = eps1 - 1/3*(eps1'*m1)*m1;             % deviatoric total strain
    s1_trial= stress_6v_to_3x3t (self,2 * mu * mI * (e1 - context.ms.eps_p));
    %     norm_s1_trial = sqrt(sum(s1_trial(1:3).^2 + 2*(s1_trial(4:6).^2)));
    norm_s1_trial = sqrt(trace(s1_trial*s1_trial));
    f1_trial = norm_s1_trial - sqrt(2/3)*sigma_y;  % trial yield f. value
    if (f1_trial > 0)
        n1 = s1_trial/norm_s1_trial;                        % unit normal to f1_trial
% How come I'm not getting the same number?
        dgamma1 = f1_trial / (2 * mu);
        theta1 = 1 - 2*mu*dgamma1/norm_s1_trial;
%         theta1 = 1 - f1_trial/norm_s1_trial;
        theta1bar =  theta1;
        n1=stress_3x3t_to_6v (self,n1);
        D = kappa*m1*m1' ...
            + 2*mu*theta1*(mI-1/3*m1*m1')...
            - 2*mu*theta1bar*n1*n1';
    else
        D = kappa*m1*m1' + 2*mu*(mI-1/3*m1*m1');
    end
    return;
% function D = tangent_moduli(self, context)
%     E = get(self.property,'E', context);;
%     nu = get(self.property,'nu', context);;
%     lambda = E * nu / (1 + nu) / (1 - 2*(nu));          % Lame constant #1
%     mu     = E / (2 * (1 + nu));                        % shear modulus
%     kappa  = E / 3 / (1 - 2*nu);                        % bulk modulus
%     mI = diag([1 1 1 0.5 0.5 0.5]);
%     m1     = [1 1 1 0 0 0]';
%     if (context.ms.dgamma > 0)
%         gradU = context.F - eye(3,3);                       % displacement gradient
%         eps1 = 1/2*(gradU + gradU');                        % total strain
%         e1   = eps1 - 1/3*trace(eps1)*eye(3,3);             % deviatoric total strain
%         ve1  = strain_3x3t_to_6v (self,e1);
%         s1_trial = 2 * mu * mI * (ve1 - context.ms.eps_p);          % trial deviatoric stress
%         norm_s1_trial = sqrt(s1_trial(1:3)'*s1_trial(1:3)+2*s1_trial(4:6)'*s1_trial(4:6));
%         n1 = s1_trial/norm_s1_trial;                        % unit normal to f1_trial
%         theta1 = 1 - 2*mu*context.ms.dgamma/norm_s1_trial;
%         theta1bar = context.ms.dgamma + theta1 -1;
%         D = kappa*m1*m1' ...
%             + 2*mu*theta1*(mI-1/3*m1*m1')...
%             - 2*mu*theta1bar*n1*n1';
%     else
%         D = kappa*m1*m1' + 2*mu*(mI-1/3*m1*m1');
%     end
%     return;    
% function D = tangent_moduli(self, context)
%     E = get(self.property,'E', context);;
%     nu = get(self.property,'nu', context);;
%     lambda = E * nu / (1 + nu) / (1 - 2*(nu));          % Lame constant #1
%     mu     = E / (2 * (1 + nu));                        % shear modulus
%     kappa  = E / 3 / (1 - 2*nu);                        % bulk modulus
%     mI = diag([1 1 1 0.5 0.5 0.5]);
%     m1     = [1 1 1 0 0 0]';
%     if (context.ms.dgamma > 0)
% %         disp(['Plastic']);
%         gradU = context.F - eye(3,3);                              % displacement gradient
%         eps1 = 1/2*(gradU + gradU');                        % total strain
%         e1   = eps1 - 1/3*trace(eps1)*eye(3,3);             % deviatoric total strain
%         ve1  = strain_3x3t_to_6v (self,e1);
%         s1_trial = 2 * mu * mI * (ve1 - context.ms.eps_p);          % trial deviatoric stress
%         norm_s1_trial = sqrt(s1_trial(1:3)'*s1_trial(1:3)+2*s1_trial(4:6)'*s1_trial(4:6));
%         n1 = s1_trial/norm_s1_trial;                        % unit normal to f1_trial
%         theta1 = 1 - 2*mu*context.ms.dgamma/norm_s1_trial;
%         D = kappa*m1*m1' + ...
%             2*mu*theta1*((mI-1/3*m1*m1') - n1*n1');
%     else
%         D = lambda * m1 * m1' + 2 * mu * mI;
%     end
%     return;
