% advance: advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where 
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step 
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string, 
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
Omegan1 = self.Omega; 
Rn1 = self.R;
t=get(self.rotint,'t');
I=self.I;
% calculate effective loads at t_n-1
tn1 = feval(torque, t, Rn1);
if (isempty(self.Alpha))
    Alphan1=I\(Rn1'*tn1 - skewmat(Omegan1)*I*Omegan1);
    self.Alpha=Alphan1;
end
for i=1:nsteps
    Psin=dt*skewmat(Omegan1+dt/2*self.Alpha);
    Rn=Rn1*rotmat(Psin);
    tn = feval(torque, t+dt, Rn);
    Omegan = I\(Rn'*(Rn1*I*Omegan1+dt/2*(tn1+tn))); % trap approx of impulse
    % swap variables for next step
    Rn1=Rn;
    self.Alpha = (Omegan-Omegan1)/dt;
    Omegan1=Omegan;
    tn1=tn;
    t=t+dt;
end
self.R = Rn; 
self.Omega = Omegan1;
self.rotint = set(self.rotint,'t',t);
retobj=self;

