% Baffled circular piston, prescribed time-dependent pressure, without absorbing
% boundary condition (the outer spherical surface has sound-hard BC, 
% that is zero velocity -- reflective surface)
rho=1.21e-3*1e-9;% mass density of air
c =343*1000;% sound speed of air, millimeters per second
bulk= c^2*rho;
P_piston=1e-3;% MPa
R=50;% length of the box, millimeters
Ro=400; % width of the box, millimeters
tol=R/10000;
nref=2;%
nlayers=48;
frequency=2000; % Hz
omega=2*pi*frequency;
dt=1/frequency/20;
tfinal=50*dt;
nsteps=round(tfinal/dt)+1;
graphics= true;% plot the solution as it is computed?

[fens,gcells]=H8_sphere(R,nref);

bg=mesh_bdry(gcells);
l=gcell_select(fens,bg,struct ('facing', true, 'direction', [1,1,1]));
% gv=drawmesh({fens,gcells1},'gcells','facecolor','white')
[fens1,gcells1] = Q4_extrude(fens,subset(bg,l),nlayers,@(xyz, layer)(R+layer/nlayers*(Ro-R))*xyz/norm(xyz));
%     gcells1=set(gcells1,'label',2);
[fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
gcells=cat(gcells1,gcells2);

options. distance=R;
options. inflate=tol;
l1=gcell_select(fens, bg, options);
options. facing=true;
options. direction=[-1,0,0];
l2=gcell_select(fens, bg, options);
piston_gcells=subset(bg,intersect(l1,l2));


gv=drawmesh({fens,mesh_bdry(gcells)},'gcells','facecolor','white','facealpha',0.2);
gv=drawmesh({fens,piston_gcells},'gv',gv,'gcells','facecolor','yellow');

boundary_gcells=mesh_bdry(gcells);
outerl=gcell_select(fens,boundary_gcells,struct ('facing', true, 'direction', [1,1,1]));

prop=property_viscous_fluid(struct('bulk',bulk,'rho',rho));
mater=mater_defor_ss_linviscel_triax (struct('property',prop));
feb = feblock_acoustics (struct (...
    'mater',mater,'gcells',gcells,...
    'integration_rule',gauss_rule(3,2)));

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
piston_fenids=connected_nodes (piston_gcells);
piston_prescribed=ones(1,length (piston_fenids));
piston_comp=[];
piston_val=piston_fenids*P_piston*sin(2*pi*frequency*0);
P = set_ebc(P, piston_fenids, piston_prescribed, piston_comp, piston_val);
P = apply_ebc (P);
P = numbereqns (P);

S = start (sparse_sysmat, get(P, 'neqns'));
S = assemble (S, acoustic_stiffness(feb, geom, P));
mS =get(S,'mat');
C = start (sparse_sysmat, get(P, 'neqns'));
C = assemble (C, acoustic_mass(feb, geom, P));
mC =get(C,'mat');

% Solve
P0 =0*P; % initially all pressure is zero
clear P
Pdd0 =0*P0; % initially all pressure acceleration is zero
vP0= gather_sysvec(P0);
vP1=0*vP0;
vPd0= 0*vP0;
vPd1=0*vPd0;
t=0;
piston_P_val=0*piston_fenids+P_piston*sin(omega*t);
    piston_Pdd_val=0*piston_fenids+(-P_piston*omega^2)*sin(omega*t);
P0 = set_ebc(P0, piston_fenids, piston_prescribed, piston_comp, piston_P_val);
P0 = apply_ebc (P0);
P1 =P0;
Pdd0 = set_ebc(Pdd0, piston_fenids, piston_prescribed, piston_comp, piston_Pdd_val);
Pdd0 = apply_ebc (Pdd0);
Pdd1 =Pdd0;

D= (2/dt)*mS +(dt/2)*mC;

if ( graphics )
    bfeb = feblock(struct ('gcells',boundary_gcells));
        gv=graphic_viewer;
end 
    
step =0;
while t <=tfinal
    step = step  +1;
    tic; disp([' Time ' num2str(t) ' (' num2str(step) '/' num2str(round(tfinal/dt)+1) ')'])
    if graphics
        Camera =1.0e+003 * [-1.4303   -3.0332   -2.0483    0.1553    0.2912    0.2533         0         0    0.0010    0.0066];
        wv = get(P1,'values');
        dcm=data_colormap(struct ('range',P_piston/2*[-1,1], 'colormap',jet));
        colors=map_data(dcm, wv);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        gv=reset (gv,[]);
        camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',0*geom,'colorfield',colorfield));
        draw_axes (gv,struct('length',R));
        pause(0.55);
    end
    t=t+dt;
    piston_P_val=0*piston_fenids+P_piston*sin(omega*t);
    piston_Pdd_val=0*piston_fenids+(-P_piston*omega^2)*sin(omega*t);
    P1 = set_ebc(P1, piston_fenids, piston_prescribed, piston_comp, piston_P_val);
    P1 = apply_ebc (P1);
    Pdd1 = set_ebc(Pdd1, piston_fenids, piston_prescribed, piston_comp, piston_Pdd_val);
    Pdd1 = apply_ebc (Pdd1);
    F = start (sysvec, get(P1, 'neqns'));
    F = assemble (F, nz_ebc_acoustic_mass_loads(feb, geom, P0+P1));
    F = assemble (F, nz_ebc_acoustic_stiffness_loads(feb, geom, Pdd0+Pdd1));
    vPd1 = D\((2/dt)*(mS*vPd0)-mC*(2*vP0+(dt/2)*vPd0)+get(F,'vec'));
    vP1 = vP0 + (dt/2)*(vPd0+vPd1);
    vP0 = vP1;
    vPd0 = vPd1;
    P1 = scatter_sysvec(P1, vP1);
    P0 = P1;
    Pdd0=Pdd1;
end