function    tilted_upsetting_aug
    disp('rectangular billet upsetting');

    % Parameters:
    E=1e3;
    nu= 0.499;
    L= 300;
    W= 300;
    H= 300;
    nsteps = 1;
    center = [L, W/2, H/2];
    travel =1.1*L/4;
    dcenter = [-travel/nsteps,0, 0];
    nedges = 8;
    penalty = 100*E;
    naugit=10;
    augtol =1/1000;
    Punchnormal=[-1, 0.1, 0.3];
    Punchnormal=Punchnormal/norm(Punchnormal);

    function  gap = get_gap (surface_data,x)
        gap = dot((x-center),Punchnormal);
        if (gap>0), gap = 0; end
    end
    function normal = get_normal(surface_data,x)
        normal=Punchnormal;
    end

    % Mesh
    %     [fens,gcells] = blockH20(L,W,H,nedges,nedges,nedges);
    %     [fens,gcells] = t4blockdel(L,W,H,nedges,nedges,nedges);
    [fens,gcells] = H8_block(L,W,H,nedges,nedges,nedges);
    bdry_gcells = mesh_bdry(gcells,  []);
    topfacelist =gcell_select(fens,bdry_gcells,struct ('box',[L L 0 W 0 H],'inflate',L/1000));
    cn = connected_nodes(subset(bdry_gcells,topfacelist));
    gap_gcells=gcellset_P1 (struct('conn',cn));
    
    % Material
    prop = property_linel_iso (struct('E',E,'nu',nu));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    % Finite element block
    feb = feblock_defor_ss_nice (struct('mater',mater,'gcells',gcells,...
        'integration_rule',tensprod_nq_rule (3),'stabfact',0.1));
    gapfeb = feblock_defor_gap (struct('gcells',gap_gcells,...
        'get_gap',@get_gap,'get_normal',@get_normal,'penalty',penalty));
    % Geometry
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));

    %     h=progressbar(mfilename);
    for step= 1:nsteps
        disp(['Step ' num2str(step)])
        center= center +dcenter;
        % Define the displacement field
        u   = clone(geom,'u');
        u   = u*0; % zero out
        % Apply EBC's
        ebc_fenids=fenode_select (fens,struct('box',[0, 0, 0, W, 0, H],'inflate',W/1000));
        ebc_prescribed=ebc_fenids*0+1;
        ebc_comp=[];% all components
        ebc_val=ebc_fenids*0;% zero displacement
        u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
        u   = apply_ebc (u);
        % Number equations
        u   = numbereqns (u);

        tic;
        K = start (sparse_sysmat, get(u, 'neqns'));
        K = assemble (K, cat(2,stiffness(feb, geom, u),stiffness(gapfeb, geom, u)));
        disp(['Stiffness matrix assembly ' num2str(toc)])
            F = start (sysvec, get(u, 'neqns'));
        lm =get(F,'vec');
        gv=reset (graphic_viewer,[]);
        augit = 1;
        while (augit<naugit)
            disp([' Iteration ' num2str(augit)])
            tic;
            F = assemble (start (sysvec, get(u, 'neqns')), gap_loads(gapfeb, geom, u));
            disp(['    Gap load assembly ' num2str(toc)])
            gl =get(F,'vec');
            tic;
            u = scatter_sysvec(u, get(K,'mat')\(-lm+gl));
            disp(['    Solve ' num2str(toc)])
            lm=lm-gl;
            % draw(feb,gv, struct ('x', geom, 'u', (0)*u,'facecolor','none','edgecolor','red'));
            % draw(feb,gv, struct ('x', geom, 'u', (1)*u,'facecolor','none','edgecolor','red'));pause(5) 
            if (norm(gl)<augtol*norm(lm)), break,end
            augit =augit+1;
        end
        geom =geom+u;% update geometry
        %         if isempty(progressbar((step/nsteps),h)), return, end;
    end
    % Plot
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=1;
    %         draw(feb,gv, struct ('x', geom, 'u', 0*u, 'facecolor','none'));
    draw(feb,gv, struct ('x', geom, 'u', (0)*u,'facecolor','red'));
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    hold on
end
