kappa=[0.2 0; 0 0.2]; % conductivity matrix
Q=0.013; % uniform heat source
graphics=1; Plot_errors=1;
Mesh_options =struct('quadratic',~true);
Targetnel=650;
convergence_rate=1.5;
Region_definition={'REGEXT 0 0 48 48',...
    ['curve 1 line 20 0 48 0'],...
    ['curve 2 line 48 0 48 48'],...
    ['curve 3 line 48 48 0 48'],...
    ['curve 4 line 0 48 0 13'],...
    ['curve 5 line 0 13 20 13'],...
    ['curve 6 line 20 13 20 0'],...
    ['subregion 1  property 1 boundary 1 2 3 4 5 6']};

mesh_size=5;
[fens,gcells,groups,edge_gcells,edge_groups]...
    =targe2_mesher(cat(2,Region_definition,{['m-ctl-point constant ' num2str(mesh_size)]}), 1.0, Mesh_options);

m=[];
for Adapt=1:3
    
    m=[m,count(gcells)]
    
    prop=property_diffusion(struct('conductivity',kappa,'source',Q));
    mater=mater_diffusion (struct('property',prop));
    feb = feblock_diffusion (struct ('mater',mater,...
        'gcells',gcells,...
        'integration_rule',tri_rule(1)));
    geom = field(struct('name',['geom'], 'dim', 2, 'fens',fens));
    theta=field(struct('name',['theta'], 'dim', 1, 'nfens',...
        get(geom,'nfens')));
    fenids=[fenode_select(fens,struct('box',[48 48 0 48],...
        'inflate', 1e-7)),fenode_select(fens,struct('box',[0 48 48 48],...
        'inflate', 1e-7)),fenode_select(fens,struct('box',[0 20 0 13],...
        'inflate', 1e-7))];
    prescribed=ones(length(fenids),1);
    comp=[];
    val=zeros(length(fenids),1)+ 30;
    theta = set_ebc(theta, fenids, prescribed, comp, val);
    theta = apply_ebc (theta);
    theta = numbereqns (theta);
    K = start (sparse_sysmat, get(theta, 'neqns'));
    K = assemble (K, conductivity(feb, geom, theta));
    F = start (sysvec, get(theta, 'neqns'));
    F = assemble(F, source_loads(feb, geom, theta));
    F = assemble(F, nz_ebc_loads_conductivity(feb, geom, theta));
    theta = scatter_sysvec(theta, get(K,'mat')\get(F,'vec'));
    
    nodal_fluxes = field_from_integration_points_spr(feb, geom, theta,  'flux', 1:2);
    elerrs = flux_L2_error (feb, geom, theta, nodal_fluxes);
    total_err=sqrt(sum(elerrs.^2));
    targeterr=sqrt(total_err^2/Targetnel); 
    
    x=get(geom,'values');
    conn=get(gcells,'conn');
    [hcurs, hests] =T3_mesh_sizes(conn,x,targeterr,elerrs,convergence_rate);
    
    % Plot
    if (graphics )
        if (Plot_errors)
            gv=graphic_viewer;
            gv=reset (gv,struct('limits', [0,48,0,48]));
            cmap=gray;cmap=cmap(end:-1:1,:);colormap(cmap);
            nvals=elerrs;
            dcm=data_colormap(struct ('range',[min(nvals),max(nvals)], 'colormap',cmap));%[min(nvals),max(nvals)]
            for i=1:count (gcells)
                color = map_data (dcm, nvals(i));
                draw(subset(gcells,i), gv, struct ('x',geom,'u',0*geom, 'facecolor',color));
            end
            view (2)
            lighting  none;
            nvalsrange=[min(nvals),max(nvals)];
            cbh=colorbar;
            set(cbh,...
                'Position',[0.8 0.15 0.05 0.7],...
                'YLim',[0,1],...
                'YTick',[0,1],...
                'YTickLabel',{[num2str(nvalsrange(1))],[num2str(nvalsrange(2))]});%{[num2str((min(nvals)))],[num2str((max(nvals)))]}
            set(get(cbh,'XLabel'),'String',['Error']);
            pause(1);
        else
            gv=graphic_viewer;
            gv=reset (gv,[]);
            T=get(theta,'values');
            dcm=data_colormap(struct('range',[min(T),max(T)],'colormap',jet));
            colorfield=field(struct ('name', ['colorfield'], 'data',...
                map_data(dcm, T)));
            geomT=field(struct ('name', ['geomT'], ...
                'data',[get(geom,'values'), get(theta,'values')]));
            draw(gcells, gv, struct ('x',geomT, 'u',0*geomT,...
                'colorfield',colorfield, 'shrink',1));
            draw(gcells, gv, struct ('x',geom, 'u',0*geom, ...
                'facecolor','none'));
            % draw_integration_points(feb, gv, struct ('x',geom,'u',0*geom, ...
            %     'theta', theta, 'scale', 35,'color','red'));
            xlabel('X [m]')
            ylabel('Y [m]')
            zlabel('Temperature [degrees C]')
            headlight(gv);
            camset (gv, [-315.9949  -97.6724  226.3628   17.7580   27.1126   12.2654         0         0    1.0000    7.6699])
        end
    end
    
    [fens,gcells,groups,edge_gcells,edge_groups] ...
        = targe2_mesher_adapt(Region_definition,conn,x,hests,1.0,Mesh_options);
    
end