u=physical_units_struct('si');

    rho_blubber=1000*u.KG/u.M^3;% mass density
    R= 1.2/2*u.M;
    nperR=4;
    kappa_blubber=3*0.18*u.W/u.M/u.K*eye(3); 
cv_blubber =3.6*u.MEGA*u.J/u.M^3/u.K;
Ti=37*u.K;
Ta=1*u.K;
dt=3600*u.SEC; % time step, try also 5.0, 2.0
theta =1.0; % generalized trapezoidal method parameter
tend= 15*24*3600*u.SEC; % length of the time interval
t=0;
tol=R/nperR/100;
online_graphics=0;

    [fens,gcells]=H8_sphere_n(R,nperR);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [-1,0,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [0,-1,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    [fens1,gcells1] = mirror_mesh(fens, gcells, [0,0,-1], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    
    gv=drawmesh({fens,gcells,'facecolor','white'},'gcells','facecolor','white','facealpha',0.1)
    
    
    prop_blubber=property_diffusion (struct('conductivity',kappa_blubber,...
    'specific_heat',cv_blubber,'source',0.0));
mater_blubber=mater_diffusion (struct('property',prop_blubber));
feb_blubber = feblock_diffusion (struct ('mater',mater_blubber,...
    'gcells',gcells,...
    'integration_rule',gauss_rule(3,2)));

boundary_gcells=mesh_bdry(gcells,[]);
bfeb = feblock_diffusion (struct ('mater',mater_blubber,...
    'gcells',boundary_gcells,...
    'integration_rule',gauss_rule(2,2)));
geom = field(struct('name',['geom'], 'dim', 3, 'fens',fens));
tempn=field(struct('name',['tempn'], 'dim', 1, 'nfens',...
    get(geom,'nfens')));
conn = connected_nodes(boundary_gcells);
tempn = set_ebc(tempn, conn, conn*0+1, [], conn*0+Ta);
tempn = apply_ebc (tempn);

tempn = numbereqns (tempn);
tempn = scatter_sysvec(tempn,gather_sysvec(tempn)*0+Ti);
 
K = start (sparse_sysmat, get(tempn, 'neqns'));
K = assemble (K, conductivity(feb_blubber, geom, tempn));
Km = get(K,'mat');
C = start (sparse_sysmat, get(tempn, 'neqns'));
C = assemble (C, capacity(feb_blubber, geom, tempn));
Cm = get(C,'mat');

if online_graphics
    gv=graphic_viewer;
    dcm=data_colormap(struct('range',[Ti,Ta],'colormap',jet));
end
minmaxT = []

while t<tend+0.1*dt % Time stepping
    if online_graphics
        set(gca,'FontSize', 14)
        T=get(tempn,'values');
        colorfield=field(struct('name',['colorfield'],'data',...
            map_data(dcm, T)));
        v=reset (gv,[]);
        draw(gcells,gv,struct('x',geom, 'u',0*geom,...
            'colorfield',colorfield, 'shrink',1.0));
        xlabel('X [mm]');        ylabel('Y [mm]');
        zlabel('Temperature [degrees C]')
        figure(gcf);
        title (['Time =' num2str(t)]); hold off; pause(0.1);
    end
    tempn1 = tempn;
    F = start (sysvec, get(tempn, 'neqns'));
    F = assemble (F, nz_ebc_loads_conductivity(feb_blubber, geom, ...
        theta*tempn1 + (1-theta)*tempn));
    F = assemble (F, nz_ebc_loads_capacity(feb_blubber, geom, ...
        (tempn1-tempn)*(1/dt)));
    Tn=gather_sysvec(tempn);
    minmaxT= [minmaxT; t/3600/24,min(Tn),max(Tn)]
    Tn1 = (1/dt*Cm+theta*Km) \ ((1/dt*Cm-(1-theta)*Km)*Tn+...
        get(F,'vec'));
    tempn = scatter_sysvec(tempn1,Tn1);
    t=t+dt;
end
figure
plot((1:size(minmaxT, 1))*dt,minmaxT,'linewidth', 3)
set(gca,'FontSize', 14); grid on
xlabel('Time in seconds')
ylabel('Temperature in degrees Celsius')