% Skin rheometer
function skin
    E=.5;
    nu=0.499;%Taylor data
    magn=0.1;
    R=2;% radius of contact, millimeters
    Ro=20; % Radius of the exposed surface, millimeters
    tol=R/10000;
    nperradius=5;%
    nlayers=8;
    graphics= true;% plot the solution as it is computed?
    
    [fens,gcells]=H8_sphere_n(R,nperradius);
    
    bg=mesh_bdry(gcells);
    l=gcell_select(fens,bg,struct ('facing', true, 'direction', [1,1,1]));
    % gv=drawmesh({fens,gcells1},'gcells','facecolor','white')
    extrusion_distance=cumsum([0,(1.25.^(0:nlayers))*R/nperradius]);
    
    [fens1,gcells1] = Q4_extrude(fens,subset(bg,l),nlayers,@(xyz, layer)(R+extrusion_distance(layer+1))*xyz/norm(xyz));
    %     gcells1=set(gcells1,'label',2);
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    
    
    [fens] = rotate_mesh(fens, [0,90/180*pi,0], [0,0,0]);
    
    [fens1,gcells1] = mirror_mesh(fens, gcells, [-1,0,0], [0,0,0], @(c)c([1, 4, 3, 2, 5, 8, 7, 6]));
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens, gcells, tol);
    gcells=cat(gcells1,gcells2);
    
    bg=mesh_bdry(gcells);
    clear options
    options. distance=R;
    options. inflate=tol;
    l1=gcell_select(fens, bg, options);
    clear options
    options. facing=true;
    options. direction=[0,0,1];
    l2=gcell_select(fens, bg, options);
    tip_gcells=subset(bg,intersect(l1,l2));
    
    gv=drawmesh({fens,mesh_bdry(gcells)},'gcells','facecolor','white','facealpha',0.2);
    gv=drawmesh({fens,tip_gcells},'gv',gv,'gcells','facecolor','yellow');
    
    outerl=gcell_select(fens,bg,struct ('facing', true, 'direction', [0,1,-1]));
    truncation_gcells=subset(bg,outerl);
    gv=drawmesh({fens,truncation_gcells},'gv',gv,'gcells','facecolor','Green');
    
    prop = property_linel_iso (struct('E',E,'nu',nu));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    feb = feblock_defor_ss_nice(struct ('mater',mater, 'gcells',gcells,...
        'integration_rule',tensprod_nq_rule(3),'stabfact', 0.1));
    
    pfeb = feblock_defor_ss (struct (...
        'mater',mater,'gcells',tip_gcells,...
        'integration_rule',gauss_rule(2,2)));
    
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    %% Define the displacement field, and zero it out
    u   = 0*geom;
    
    %% Apply the EBC''s
    ebc_fenids=connected_nodes(truncation_gcells);
    ebc_prescribed=ones(length(ebc_fenids),1);
    ebc_comp=[];
    ebc_val=zeros(length(ebc_fenids),1);
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=fenode_select(fens,struct('box',[-inf inf 0 0 -inf inf],'inflate',tol));
    ebc_prescribed=ones(length(ebc_fenids),1);
    ebc_comp=ones(length(ebc_fenids),1)*2;
    ebc_val=zeros(length(ebc_fenids),1);
    u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    u   = apply_ebc (u);
    
    %% Number the equations
    u   = numbereqns (u);
    
    %% Assemble the stiffness matrix
    K = start (sparse_sysmat, get(u, 'neqns'));
    K = assemble (K, stiffness(feb, geom, u));
    
    %% Assemble the load vector
    fi= force_intensity(struct ('magn',[magn;0;0; ]));
    F = start (sysvec, get(u, 'neqns'));
    F = assemble (F, distrib_loads(pfeb, geom, u, fi,2));
    
    
    %% Solve K*u=F, Transfer the solution to the field u
    u = scatter_sysvec(u, K\F);
    
    %% Plot the displacement field u
    if graphics
        gv=graphic_viewer;
        gv=reset (gv,[]);
        scale=1;
        sfeb = feblock_defor_ss (struct (...
            'mater',mater,'gcells',mesh_bdry(gcells),...
            'integration_rule',gauss_rule(2,2)));
        %         draw(sfeb,gv, struct ('x', geom,'u',scale*u, 'facecolor','none'));
        Uv=get(u,'values');
        Umag = real (sqrt(Uv(:,1).^2+Uv(:,2).^2+Uv(:,3).^2));
        dcm=data_colormap(struct ('range',[0,max(Umag)], 'colormap',jet));
        colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, Umag)));
        draw(feb, gv, struct ('x', geom,'u', scale*u, 'colorfield',colorfield));%, 'shrink',0.9
        frame=1;
        for xscale=scale*sin((0:1:84)/21*2*pi)
            gv=reset (gv,[]);
            camset(gv,[ -105.9320 -136.5231   99.5394    2.9230 5.3395   -3.6989         0         0 1.0000    2.0018]);
            draw(sfeb,gv, struct ('x', geom, 'u', xscale*u,'colorfield',colorfield));
            pause(0.005);
            gif_animation_add_frame(gcf,frame,[mfilename '-movie.gif'],80);
            frame =frame+1;
        end
    end
end