
function L
% Parameters:
E=1e7;
nu=0.3;
graphics=0;
scale = 1;
W=1; H=2;
n=6;

% Mesh
[fens,gcells] = Q4_block(W,H,1,1,struct( 'other_dimension', 1 ));
for n=1:1:n
    [fens,gcells]=Q4_refine(fens,gcells);
end
% Material
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_biax (struct('property',prop, ...
    'reduction','strain'));
% Finite element block
feb = feblock_defor_ss (struct ('mater',mater, 'gcells',gcells,...
    'integration_rule',gauss_rule (2, 2)));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 2, 'fens',fens));
% Define the displacement field
u   = clone(geom,'u');
u   = u*0; % zero out
% Apply EBC's
ebc_fenids=fenode_select (fens,struct('box',[0 0 0 H],'inflate', 0.0001));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=1*ones(1,length (ebc_fenids));
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select (fens,struct('box',[0 W 0 0],'inflate', 0.0001));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=2*ones(1,length (ebc_fenids));
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select (fens,struct('box',[0 W H H],'inflate', 0.0001));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=2*ones(1,length (ebc_fenids));
ebc_val=ebc_fenids*0+0.25;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);
% Number equations
u   = numbereqns (u);
% Assemble the system matrix
ems = stiffness(feb, geom, u);
K = sparse_sysmat;
K = start (K, get(u, 'neqns'));
K = assemble (K, ems);
K = finish (K);
% Load
F = sysvec;
F = start (F, get(u, 'neqns'));
evs = nz_ebc_loads(feb, geom, u);
F = assemble (F, evs);
F = finish(F);
% Solve
u = scatter_sysvec(u, K\F);
Energy=get(F, 'vec' )'*gather_sysvec(u)/2
ufree=u;
ufree = set_ebc(ufree, 1:count(fens), 0, [], 0);
ufree   = apply_ebc (ufree);
ufree  = numbereqns (ufree);
ems = stiffness(feb, geom, ufree);
K = sparse_sysmat;
K = start (K, get(ufree, 'neqns'));
K = assemble (K, ems);
StrainEnergy=gather_sysvec(ufree)'*get(K, 'mat' )*gather_sysvec(ufree)/2

% Plot
if (graphics )
gv=graphic_viewer;
gv=reset (gv,struct ('limits', [0, W, 0, H*1.3]));
cmap=jet;
fld = field_from_integration_points(feb, geom, u, [], 'Cauchy', 4);
nvals=get(fld,'values');
dcm=data_colormap(struct ('range',[min(nvals),max(nvals)], 'colormap',cmap));%[min(nvals),max(nvals)]
colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
draw(feb, gv, struct ('x',geom,'u', 0*u, 'facecolor','none', 'shrink',1));
draw(feb, gv, struct ('x',geom,'u', scale*u, 'colorfield',colorfield, 'shrink',1));
lighting  none;
title (['Stress']);
colormap(cmap);
cbh=colorbar;
set(cbh,...
    'Position',[0.76 0.15 0.1 0.7],...
    'YLim',[0,1],...
    'YTick',[0,1],...
    'YTickLabel',{[num2str((min(nvals)))],[num2str((max(nvals)))]});%{[num2str((min(nvals)))],[num2str((max(nvals)))]}
set(get(cbh,'XLabel'),'String','\sigma_{xy}');
view (2)
end
end