function Cylinder_free
disp('Example: Free vibration of a thin cylindrical shell');
[uSEC,uMIN,uHR,uDAY,uWK,uMONTH,uYR,uFT,uSLUG,uRAN,...
    uM,uKG,uK,uCM,uGM,uDEG,uRAD,uREV,...
    uIN,uNMI,uYD,uMM,uMILE,uG,uLB,uNT,uOZ,uLBM,uKT,...
    uMPH,uPSI,uPA,uBAR,uATM,uTORR,ummHG,uJ,...
    uCAL,uMEV,uERG,uBTU,uW,uMW,uHP,uA,uV,...
    uHZ,uRPS,uRPM,uL,uGAL,uGPM,...
    uMICRO,uKILO,uMILLI,uMEGA,uGIGA,uTERA]=physical_units;
% Studied In C:\Users\pkrysl\Documents\Research\Scattering\Report_October_2011
E= 205e9*uPA;% [MPa]
rho=7850*uKG/uM^3;%  [1000*kg/mm^3]
nu=0.33;%
rA=2*pi ; Thickness= 0.01*uM; Length= 1.0*uM; nc= 15;nt= 1;na=10;
iR = 0.5*uM;
scale=0.15;
neigvs = 8;
graphics=1;

% Mesh

[fens,gcells]=H8_block(rA, Thickness, Length, nc,nt,na);
%[fens,gcells]=H8_blockx(linspace(0,rA,nc), linspace(0,Thickness,nt+1),...
    %[linspace(0,3/4*Length,na/2),linspace((1+1/na)*3/4*Length,Length,na/2)]);
[fens,gcells] = H8_to_H20(fens,gcells);
xyz=get (fens,'xyz');
for i=1:count (fens)
    x=xyz(i,1); y=xyz(i,2); z=xyz(i,3);
    xyz(i,:)= [0, y-iR, z];
    xyz(i,:) = xyz(i,:)*rotmat([0, 0, -x])+ [0,iR,0];
end
fens=set(fens,'xyz', xyz);
[fens,gcells] = merge_nodes(fens, gcells,  Thickness/100);
% drawmesh({fens,gcells})
% Material
prop = property_linel_iso(struct('E',E,'nu',nu));
mater = mater_defor_ss_linel_triax(struct('property',prop));
% Finite element block
feb = feblock_defor_ss (struct ('mater',mater, 'gcells',gcells,...
    'integration_rule',gauss_rule(3,2)));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
limits =reshape([min(get(geom,'values'));max(get(geom,'values'))], 6, 1);
% Define the displacement field
u   = 0*geom; % zero out


% Number equations
u   = numbereqns (u);


disp(['Mass ' num2str(get(u, 'neqns')) 'x' num2str(get(u, 'neqns')) ' assembly']);

M = start (sparse_sysmat, get(u, 'neqns'));
M = assemble (M, mass(feb, geom, u));
Mm =(get(M,'mat')); clear M

% Assemble the system matrix
disp(['Stiffness ' num2str(get(u, 'neqns')) 'x' num2str(get(u, 'neqns')) ' assembly']);
tic
K = start (sparse_chol_sysmat, get(u, 'neqns'));
K = assemble (K, stiffness(feb, geom, u));

toc
clear evs

%
disp(['EP']);
OmegaShift=(2*pi*100);
neigvs=6+neigvs;% to account for free-floating structure
clear options
options.issym= true;
% options.tol=1e-12;
% options.maxit= 500;
options.disp= 2;
K = assemble_diag (K, OmegaShift^2*diag(Mm));
[W,Omega]=eigs(@(b) (K\b),get(u, 'neqns'),Mm,neigvs,'SM',options);
% Undo the shift
if (min(real(diag(Omega)))<0)
    error('Shift might have led to missing low eigenvalues');
end
Omega =diag(diag(Omega)-OmegaShift^2);
[Ignore,ix] =sort (diag(Omega));
Omega =Omega(ix,ix);
W =W (:,ix);


disp(['EP Done']);


for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
end


% Plot
if graphics
    Camera =1.0e+003 * [   -1.0612
        -0.8377
        1.0948
        -0.0775
        0.0025
        0.0093
        0
        0
        0.0010
        0.0033];
    xyz=get(geom,'values');
    box =reshape([min(xyz);max(xyz)],1,6);
    annloc=[0.8*max(box(1:2)), min(box(3:4)), mean(box(5:6))];
    bg=mesh_bdry(gcells);
    bfeb = feblock(struct ('gcells',bg));
    gv=graphic_viewer;
    gv=reset (gv,struct('limits',  limits));
    scale=1.65;
    w=clone(u,'w'); % make a copy of u
    for i=7:neigvs
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]']);
        %         clf;
        w = scatter_sysvec(w, W(:,ix(i)));
        w = w*(1/norm(w ));
        wv = get(w,'values');
        wmag = real (sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2));
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        for xscale=scale*sin((0:1:42)/21*2*pi)
            gv=reset (gv,[]);
            %             camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom, 'u', xscale*w,'colorfield',colorfield));
            draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) ' [Hz]'],...
                struct('fontsize', 18));
            draw_axes (gv,struct('length',Length/20 ));
            pause(0.005);
        end
        %     saveas(gcf, ['twist_t4-' num2str(i) '.png'], 'png');
        pause(2); % next eigenvector
        gv=reset (gv,[]);
        %         camset(gv,Camera);
        draw(bfeb,gv, struct ('x', geom,'u',+scale*w,'colorfield',colorfield));
        draw(bfeb,gv, struct ('x', geom,'u',0*w,'facecolor','yellow','edgecolor',.53+[0, 0, 0],'alpha', 0.2));
        draw_text(gv, annloc,['  Eigenvector ' num2str(i) '\newline     ' num2str(sqrt(Omega(ix(i),ix(i)))/2/pi) '  [Hz]'],...
            struct('fontsize', 18));
        draw_axes (gv,struct('length',Length/20 ));
        pause(2.55);
        
    end
end