disp('Wave propagation along a rod');
% clear all classes
graphics = ~false;
% Parameters:
E=100;
% nu=0.49993;
nu=0.3;
L = 200;
W = 4;
H = 4;
vmag = 0.1;
tfinal = 300;
scale=100;
stressscale=scale/20;

% [fens,gcells] = block(L,W,H, 25, 2,2);
[fens,gcells] = block(L,W,H, 50, 1,1);
% [fens,gcells] = block(L,W,H, 5, 1,1);
% [fens,gcells] = H8_to_H20(fens,gcells);
ir =gauss_rule (3,2);
% Material
prop = property_linel_iso (struct('E',E,'nu',nu));
mater = mater_defor_neohook(struct('property',prop));
% Finite element block
feb = feblock_defor_nonlinear (struct ('mater',mater, 'gcells',gcells, 'integration_rule',ir));
% feb = feblock_defor_nonlinear_nice (struct ('mater',mater,'gcells',gcells, 'integration_rule',ir));
% Geometry
geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
% Define the displacement field
u   = 0*geom; % zero out
% Apply EBC's

ebc_fenids=fenode_select (fens,struct('box',[L,L,-Inf,Inf,-Inf,Inf],'inflate',44/10000));
ebc_prescribed=ones(1,length (ebc_fenids));
ebc_comp=[];
ebc_val=ebc_fenids*0;
u   = set_ebc(u, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
u   = apply_ebc (u);

% Number equations
u   = numbereqns (u);




        
% Initial conditions
v = u;% copy everything from displacement field to velocity
fenids=fenode_select (fens,struct('box',[0,0,-Inf,Inf,-Inf,Inf],'inflate',44/10000));
for j=1:length(fenids)
    xy= get (fens(fenids(j)),'xyz');
    v = scatter(v,fenids(j),[1,0,0]*vmag);
end
% Assemble the system matrix
ess=stiffness(feb, geom, u,u);% Current material stiffness
M = start (sparse_sysmat, get(u, 'neqns'));
ems=mass(feb, geom, u);
Max_lambda = 0;
for j=1:length(ems)
    Me=get(ems(j),'mat');
    Me =diag(sum(Me));
    ems(j)=set(ems(j),'mat',Me);
    Ke=get(ess(j),'mat');
    [V,D] = eigs (Ke, Me, 1);
    Max_lambda = max( [Max_lambda, D(1)]);
end
M = assemble (M, ems);
dt= 0.99* 2/sqrt(Max_lambda)
% Solve
Mmat = get(M,'mat');
U0 = gather_sysvec(u);
V0 = gather_sysvec(v);
u0 = scatter_sysvec(u, U0);
t=0;
F = start (sysvec, get(u, 'neqns'));
[feb,evs] = restoring_force(feb,geom,u0,u0);       % Internal forces
F = assemble (F, evs);
A0 = Mmat\(get(F,'vec'));
if graphics
    gv=graphic_viewer;
    gv=reset (gv,struct ('limits',[0 17 -26 26]));
end
ngraphics=  max (round(2*(1e-2)/dt),1);
igraphics =ngraphics-1;
snapshot =0;
while t <tfinal
    feb1 = feb;
    t=t+dt 
    U1 = U0 +dt*V0+(dt^2)/2*A0;
    u1 = scatter_sysvec(u, U1);
    F = start (sysvec, get(u, 'neqns'));
    [feb1,evs] = restoring_force(feb1,geom,u1,u);       % Internal forces
    F = assemble (F, evs);
    A1 = Mmat\(get(F,'vec'));
    V1 = V0 +dt/2* (A0+A1);
    U0 = U1;
    V0 = V1;
    A0 = A1;
    u0 = u1;
    igraphics=igraphics+1;
    if graphics & igraphics==ngraphics
        clear(gv); gv=reset (gv,struct ('limits',[0-W L+W 0-W W+W 0-W H+W]));
        u = u1;
        %         draw(feb,gv, struct ('x', geom, 'u', scale*u,...
        %             'facecolor','red'));
         nvals=get(slice(u, 1,'u'),'values');
        nvalsrange=[-0.025, 0.025]%[min(nvals),max(nvals)]
        dcm=data_colormap(struct ('range',nvalsrange, 'colormap',jet));
        colorfield=field(struct ('name', ['colorfield'], 'data',map_data(dcm, nvals)));
        draw(feb,gv, struct ('x', geom, 'u', scale*u,...
            'colorfield',colorfield));%,'edgecolor','none'
        pause(0.01);
        %         saveas(gcf, [mfilename '-33-' num2str(snapshot) '.png'], 'png');
        snapshot = snapshot +1;
        igraphics= 0;
    end
end
