% Mirror a 2-D mesh in a plane given by its normal and one point. 
%
% function [fens1,gcells1] = mirror_mesh(fens, gcells, Normal, Point)
%
% fens, gcells= mesh, 
% Normal, Point= 2-D arrays
%  
% Warning: The code to relies on the numbering of the cells: two reverse
% the orientation of the mirrored cells, the connectivity is listed in
% inverse order.   If the mirrored cells do not comply, their areas will
% come out negative.
% 
function [fens1,gcells1] = mirror_mesh(fens, gcells, Normal, Point, renumb)
    Normal = Normal/norm (Normal);
    fens1 =fens;
    xyz =get(fens1,'xyz');
    for i=1:count(fens1)
        xyz(i,:) =xyz(i,:)-2*dot(Normal,xyz(i,:))*Normal;
    end
    fens1 =set(fens1,'xyz',xyz);
    % Reconnect the cells
    if (~exist('renumb','var'))
        renumb=@(conn)conn(end:-1:1);
    end
    gcells1=gcells;
    conns=get(gcells1,'conn');
    for i=1:size(conns,1)
        conns(i,:)=renumb(conns(i,:));
    end
    gcells1=set(gcells1,'conn',conns);
end
