function u=physical_units_struct(System_of_units,base_time_units)
% physical_units - a function to create unit conversion factors
%
% function u=physical_units_struct(System_of_units,base_time_units)
%     
% 
% Inputs:
% --System_of_units defaults to SI
% if System_of_units=='US', basic assumed units are American Engineering:
% LENGTH=uFT, TIME=uSEC, MASS=uSLUG TEMP=uRAN FORCE=uLB
% if System_of_units=='CGS', basic assumed units are Centimeter,Gram,Second:
% LENGTH=uCM, TIME=uSEC, MASS=uGM TEMP=uK FORCE=uDYNE
% otherwise, basic assumed units are SI:
% LENGTH=uM , TIME=uSEC, MASS=uKG   TEMP=uK   FORCE=uNT
% --base_time_units defaults to SEC
%
% Outputs:
%    u = struct with fields      
%         uSEC,uMIN,uHR,uDAY,uWK,uMONTH,uYR,uFT,uSLUG,uRAN,...
%         uM,uKG,uK,uCM,uGM,uDEG,uREV,...
%         uIN,uNMI,uYD,uMM,uMILE,uG,uLB,uNT,uOZ,uLBM,uKT,...
%         uMPH,uPSI,uPA,uBAR,uATM,uTORR,ummHG,uJ,...
%         uCAL,uMEV,uERG,uBTU,uW,uMW,uHP,uA,uV,...
%         uHZ,uRPS,uRPM,uL,uGAL,uGPM
%
% uSEC = second
% % time conversions
% uMIN = Minute
% uHR = hour
% uDAY = Day
% uWK = Week
% uMONTH = Month
% uYR = Year
% uCOUL=Coulomb (charge);
% uRAD=Radian (angle)
% % prefixes
% uMICRO=10^(-6);
% uMILLI=10^(-3);
% uKILO=10^3;
% uMEGA=10^6;
% uGIGA=10^9;
% uTERA=10^12;
% uDEG=Degree (angle)
% uREV=Measure of angles in terms of revolutions
% uIN=Inch
% uNMI=Nautical mile
% uCM=Centimeter
% uYD=Yard
% uMM=Millimeter
% uMILE=Mile
% uG=Gravity acceleration
% uLB=Pound
% uNT=newton
% uOZ=ounce
% uLBM= Pound of mass
% uKT=knot
% uMPH=mile per hour
% uPSI=Pound per square inch
% uPA= Pascal
% uBAR=Bar
% uATM=Atmosphere
% uTORR=torr
% ummHG=mm Hg
% uJ= Joule
% uCAL = Calorie
% uMEV = Mega electron volt
% uERG = erg
% uBTU = BTU
% uW=watt
% uMW = Megawatt
% uHP=Horsepower
% uA = ampere
% uV = volt
% uHZ = Frequency (Hertz)
% uRPS = Revolutions per second
% uRPM =  Revolutions per minute
% uL = Liter
% uGAL = Gallon
% uGPM = Gallons per minute 
% 
if (~exist('System_of_units'))
    System_of_units='SI';%
end
if (~exist('base_time_units'))
    base_time_units='SEC';%
end
[uSEC,uMIN,uHR,uDAY,uWK,uMONTH,uYR,uFT,uSLUG,uRAN,...
    uM,uKG,uK,uCM,uGM,uDEG,uRAD,uREV,...
    uIN,uNMI,uYD,uMM,uMILE,uG,uLB,uNT,uOZ,uLBM,uKT,...
    uMPH,uPSI,uPA,uBAR,uATM,uTORR,ummHG,uJ,...
    uCAL,uMEV,uERG,uBTU,uW,uMW,uHP,uA,uV,...
    uHZ,uRPS,uRPM,uL,uGAL,uGPM,...
    uMICRO,uKILO,uMILLI,uMEGA,uGIGA,uTERA]=physical_units(System_of_units,base_time_units);
 
% Check the number of output arguments and report error if different from
% that expected.
if (nargout ~=1)
    error('Mismatched number of output arguments');
end
for j={'uSEC','uMIN','uHR','uDAY','uWK','uMONTH','uYR','uFT','uSLUG','uRAN',...
    'uM','uKG','uK','uCM','uGM','uDEG','uRAD','uREV',...
    'uIN','uNMI','uYD','uMM','uMILE','uG','uLB','uNT','uOZ','uLBM','uKT',...
    'uMPH','uPSI','uPA','uBAR','uATM','uTORR','ummHG','uJ',...
    'uCAL','uMEV','uERG','uBTU','uW','uMW','uHP','uA','uV',...
    'uHZ','uRPS','uRPM','uL','uGAL','uGPM',...
    'uMICRO','uKILO','uMILLI','uMEGA','uGIGA','uTERA'}
u.(j{1})=eval (j{1});
n=j{1};
u.(n(2:end))=eval (j{1});
end